@@ -1329,9 +1329,6 @@ set_up_automatic_login_session (GdmManager *manager,
         GdmSession *session;
         gboolean is_initial;
 
-        g_object_set (G_OBJECT (display), "session-class", "user", NULL);
-        g_object_set (G_OBJECT (display), "session-type", NULL, NULL);
-
         /* 0 is root user; since the daemon talks to the session object
          * directly, itself, for automatic login
          */
@@ -1517,11 +1514,14 @@ create_display_for_user_session (GdmManager *self,
                                  const char *session_id)
 {
         GdmDisplay *display;
+        /* at the moment we only create GdmLocalDisplay objects on seat0 */
+        const char *seat_id = "seat0";
 
         display = gdm_local_display_new ();
 
         g_object_set (G_OBJECT (display),
                       "session-class", "user",
+                      "seat-id", seat_id,
                       "session-id", session_id,
                       NULL);
         gdm_display_store_add (self->priv->display_store,
@@ -1597,7 +1597,13 @@ on_start_user_session (StartUserSessionOperation *operation)
                 g_object_set_data (G_OBJECT (display), "gdm-embryonic-user-session", NULL);
                 g_object_set_data (G_OBJECT (operation->session), "gdm-display", NULL);
                 create_embryonic_user_session_for_display (operation->manager, display, allowed_uid);
-                g_object_unref (display);
+
+                if (g_strcmp0 (operation->service_name, "gdm-autologin") == 0) {
+                        /* remove the unused prepared greeter display since we're not going
+                         * to have a greeter */
+                        gdm_display_store_remove (self->priv->display_store, display);
+                        g_object_unref (display);
+                }
 
                 /* Give the user session a new display object for bookkeeping purposes */
                 session_id = gdm_session_get_conversation_session_id (operation->session,
