@@ -36,6 +36,7 @@
 #include <glib-object.h>
 
 #include "gdm-display.h"
+#include "gdm-launch-environment.h"
 #include "gdm-xdmcp-display.h"
 
 #include "gdm-common.h"
@@ -51,6 +52,7 @@ struct GdmXdmcpDisplayPrivate
 {
         GdmAddress             *remote_address;
         gint32                  session_number;
+        guint                   connection_attempts;
 };
 
 enum {
@@ -59,9 +61,10 @@ enum {
         PROP_SESSION_NUMBER,
 };
 
+#define MAX_CONNECT_ATTEMPTS  10
+
 static void     gdm_xdmcp_display_class_init    (GdmXdmcpDisplayClass *klass);
 static void     gdm_xdmcp_display_init          (GdmXdmcpDisplay      *xdmcp_display);
-static void     gdm_xdmcp_display_finalize      (GObject              *object);
 
 G_DEFINE_ABSTRACT_TYPE (GdmXdmcpDisplay, gdm_xdmcp_display, GDM_TYPE_DISPLAY)
 
@@ -81,50 +84,6 @@ gdm_xdmcp_display_get_remote_address (GdmXdmcpDisplay *display)
         return display->priv->remote_address;
 }
 
-static gboolean
-gdm_xdmcp_display_create_authority (GdmDisplay *display)
-{
-        g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
-
-        return GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->create_authority (display);
-}
-
-static gboolean
-gdm_xdmcp_display_add_user_authorization (GdmDisplay *display,
-                                          const char *username,
-                                          char      **filename,
-                                          GError    **error)
-{
-        return GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->add_user_authorization (display, username, filename, error);
-}
-
-static gboolean
-gdm_xdmcp_display_remove_user_authorization (GdmDisplay *display,
-                                             const char *username,
-                                             GError    **error)
-{
-        return GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->remove_user_authorization (display, username, error);
-}
-
-static gboolean
-gdm_xdmcp_display_manage (GdmDisplay *display)
-{
-        g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
-
-        GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->manage (display);
-
-        return TRUE;
-}
-
-static gboolean
-gdm_xdmcp_display_unmanage (GdmDisplay *display)
-{
-        g_return_val_if_fail (GDM_IS_DISPLAY (display), FALSE);
-
-        GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->unmanage (display);
-        return TRUE;
-}
-
 static void
 _gdm_xdmcp_display_set_remote_address (GdmXdmcpDisplay *display,
                                        GdmAddress      *address)
@@ -185,23 +144,75 @@ gdm_xdmcp_display_get_property (GObject    *object,
         }
 }
 
-static void
-gdm_xdmcp_display_get_timed_login_details (GdmDisplay *display,
-                                           gboolean   *enabledp,
-                                           char      **usernamep,
-                                           int        *delayp)
+static gboolean
+gdm_xdmcp_display_prepare (GdmDisplay *display)
 {
-        *enabledp = FALSE;
-        *usernamep = g_strdup ("");
-        *delayp = 0;
-        gboolean allow_remote_autologin;
+        GdmXdmcpDisplay *self = GDM_XDMCP_DISPLAY (display);
+        GdmLaunchEnvironment *launch_environment;
+        char          *display_name;
+        char          *seat_id;
+        char          *hostname;
 
-        allow_remote_autologin = FALSE;
-        gdm_settings_direct_get_boolean (GDM_KEY_ALLOW_REMOTE_AUTOLOGIN, &allow_remote_autologin);
+        launch_environment = NULL;
+        display_name = NULL;
+        seat_id = NULL;
+        hostname = NULL;
 
-        if ( allow_remote_autologin ) {
-                GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->get_timed_login_details (display, enabledp, usernamep, delayp);
+        g_object_get (self,
+                      "x11-display-name", &display_name,
+                      "seat-id", &seat_id,
+                      "remote-hostname", &hostname,
+                      "launch-environment", &launch_environment,
+                      NULL);
+
+        if (launch_environment == NULL) {
+                launch_environment = gdm_create_greeter_launch_environment (display_name,
+                                                                            seat_id,
+                                                                            NULL,
+                                                                            hostname,
+                                                                            FALSE);
+                g_object_set (self, "launch-environment", launch_environment, NULL);
+                g_object_unref (launch_environment);
         }
+
+        if (!gdm_display_create_authority (display)) {
+                g_warning ("Unable to set up access control for display %s",
+                           display_name);
+                return FALSE;
+        }
+
+        return GDM_DISPLAY_CLASS (gdm_xdmcp_display_parent_class)->prepare (display);
+}
+
+static gboolean
+idle_connect_to_display (GdmXdmcpDisplay *self)
+{
+        gboolean res;
+
+        self->priv->connection_attempts++;
+
+        res = gdm_display_connect (GDM_DISPLAY (self));
+        if (res) {
+        } else {
+                if (self->priv->connection_attempts >= MAX_CONNECT_ATTEMPTS) {
+                        g_warning ("Unable to connect to display after %d tries - bailing out", self->priv->connection_attempts);
+                        gdm_display_unmanage (GDM_DISPLAY (self));
+                        return FALSE;
+                }
+                return TRUE;
+        }
+
+        return FALSE;
+}
+
+static void
+gdm_xdmcp_display_manage (GdmDisplay *display)
+{
+        GdmXdmcpDisplay *self = GDM_XDMCP_DISPLAY (display);
+
+        g_timeout_add (500, (GSourceFunc)idle_connect_to_display, self);
+
+        g_object_set (G_OBJECT (self), "status", GDM_DISPLAY_MANAGED, NULL);
 }
 
 static void
@@ -212,14 +223,9 @@ gdm_xdmcp_display_class_init (GdmXdmcpDisplayClass *klass)
 
         object_class->get_property = gdm_xdmcp_display_get_property;
         object_class->set_property = gdm_xdmcp_display_set_property;
-        object_class->finalize = gdm_xdmcp_display_finalize;
 
-        display_class->create_authority = gdm_xdmcp_display_create_authority;
-        display_class->add_user_authorization = gdm_xdmcp_display_add_user_authorization;
-        display_class->remove_user_authorization = gdm_xdmcp_display_remove_user_authorization;
+        display_class->prepare = gdm_xdmcp_display_prepare;
         display_class->manage = gdm_xdmcp_display_manage;
-        display_class->unmanage = gdm_xdmcp_display_unmanage;
-        display_class->get_timed_login_details = gdm_xdmcp_display_get_timed_login_details;
 
         g_type_class_add_private (klass, sizeof (GdmXdmcpDisplayPrivate));
 
@@ -247,20 +253,35 @@ static void
 gdm_xdmcp_display_init (GdmXdmcpDisplay *xdmcp_display)
 {
 
+        gboolean allow_remote_autologin;
+
         xdmcp_display->priv = GDM_XDMCP_DISPLAY_GET_PRIVATE (xdmcp_display);
+
+        allow_remote_autologin = FALSE;
+        gdm_settings_direct_get_boolean (GDM_KEY_ALLOW_REMOTE_AUTOLOGIN, &allow_remote_autologin);
+
+        g_object_set (G_OBJECT (xdmcp_display), "allow-timed-login", allow_remote_autologin, NULL);
 }
 
-static void
-gdm_xdmcp_display_finalize (GObject *object)
+GdmDisplay *
+gdm_xdmcp_display_new (const char *hostname,
+                       int         number,
+                       GdmAddress *address,
+                       gint32      session_number)
 {
-        GdmXdmcpDisplay *xdmcp_display;
+        GObject *object;
+        char    *x11_display;
 
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (GDM_IS_XDMCP_DISPLAY (object));
+        x11_display = g_strdup_printf ("%s:%d", hostname, number);
+        object = g_object_new (GDM_TYPE_XDMCP_DISPLAY,
+                               "remote-hostname", hostname,
+                               "x11-display-number", number,
+                               "x11-display-name", x11_display,
+                               "is-local", FALSE,
+                               "remote-address", address,
+                               "session-number", session_number,
+                               NULL);
+        g_free (x11_display);
 
-        xdmcp_display = GDM_XDMCP_DISPLAY (object);
-
-        g_return_if_fail (xdmcp_display->priv != NULL);
-
-        G_OBJECT_CLASS (gdm_xdmcp_display_parent_class)->finalize (object);
+        return GDM_DISPLAY (object);
 }
